#include "FlowField.h"

FlowField::FlowField(void)
{
	m_Left=0;
	m_Right=0;
	m_Top=0;
	m_Bottom=0;
}

FlowField::FlowField(const FlowField& other)
{
	copyData(other);
}

FlowField::~FlowField(void)
{
}

void FlowField::copyData(const FlowField &other)
{
	//printf("FlowField being copied\n");

	m_Vx=other.m_Vx;
	m_Vy=other.m_Vy;
	m_OpticalFlowPara=other.m_OpticalFlowPara;
	m_AlgorithmPara=other.m_AlgorithmPara;
	m_Left=other.m_Left;
	m_Top=other.m_Top;
	m_Right=other.m_Right;
	m_Bottom=other.m_Bottom;
}

FlowField& FlowField::operator=(const FlowField& other)
{
	copyData(other);
	return *this;
}

void FlowField::fromDoublePrecision(const DImage &vx, const DImage &vy)
{
	m_Vx.fromDoublePrecision(vx);
	m_Vy.fromDoublePrecision(vy);
}

void FlowField::toDoublePrecision(DImage &vx, DImage &vy) const
{
	m_Vx.toDoublePrecision(vx);
	m_Vy.toDoublePrecision(vy);
}

void FlowField::toFloatPrecision(FImage &vx, FImage &vy) const
{
	m_Vx.toFloatPrecision(vx);
	m_Vy.toFloatPrecision(vy);
}


void FlowField::getFlow(int x, int y, float *vx, float *vy)
{
	*vx = m_Vx.val(x, y);
	*vy = m_Vy.val(x, y);
}

//----------------------------------------------------------------------------------------
// functions for file IO
//----------------------------------------------------------------------------------------
bool FlowField::writeFlowField(QFile &file) const
{
	file.write((char *)&m_Left,sizeof(int));
	file.write((char *)&m_Top,sizeof(int));
	file.write((char *)&m_Right,sizeof(int));
	file.write((char *)&m_Bottom,sizeof(int));
	if(!m_OpticalFlowPara.writeOpticalFlowPara(file))
		return false;
	if(!m_AlgorithmPara.writeAlgorithmPara(file))
		return false;
	if(!m_Vx.writeIntegerImage(file))
		return false;
	if(!m_Vy.writeIntegerImage(file))
		return false;
	return true;
}

bool FlowField::readFlowField(QFile &file)
{
	file.read((char *)&m_Left,sizeof(int));
	file.read((char *)&m_Top,sizeof(int));
	file.read((char *)&m_Right,sizeof(int));
	file.read((char *)&m_Bottom,sizeof(int));
	if(!m_OpticalFlowPara.readOpticalFlowPara(file))
		return false;
	if(!m_AlgorithmPara.readAlgorithmPara(file))
		return false;
	if(!m_Vx.readIntegerImage(file))
		return false;
	if(!m_Vy.readIntegerImage(file))
		return false;
	return true;
}