#include <QtGui>
#include "FlowFieldViewer.h"

FlowFieldViewer::FlowFieldViewer(QWidget *parent, Qt::WFlags flags)
	: QWidget(parent, flags)
{
	//setMinimumSize(800,650);

	scrollArea=new QScrollArea;
	pImageViewer=new ImageViewer;
	pImageViewer->setMaxScale(4);
	pImageViewer->setMinScale(0.6);
	scrollArea->setWidget(pImageViewer);

	createActions();
	toolBar->setMaximumHeight(36);

	vLayout=new QVBoxLayout;
	vLayout->addWidget(toolBar);
	vLayout->addWidget(scrollArea);

	setLayout(vLayout);
	setWindowTitle(tr("Flow field viewer"));

	genStrokes=false;
}

FlowFieldViewer::~FlowFieldViewer(void)
{
}

void FlowFieldViewer::resizeEvent(QResizeEvent *event)
{
	pImageViewer->setParentSize(scrollArea->frameSize());
}

void FlowFieldViewer::createActions()
{
	 //-----------------------------------------------------------------------------------------------------------------
	 // the actions of zoom in, zoom out and set to normal size
     zoomInAct = new QAction(tr("Zoom &In"), this);
     zoomInAct->setEnabled(true);
         zoomInAct->setIcon(QIcon("icons/zoom_in.png"));
     connect(zoomInAct, SIGNAL(triggered()), pImageViewer, SLOT(zoomIn()));
	 connect(pImageViewer,SIGNAL(setZoomIn(bool)),zoomInAct,SLOT(setEnabled(bool)));

     zoomOutAct = new QAction(tr("Zoom &Out"), this);
     zoomOutAct->setEnabled(true);
         zoomOutAct->setIcon(QIcon("icons/zoom_out.png"));
     connect(zoomOutAct, SIGNAL(triggered()), pImageViewer, SLOT(zoomOut()));
	 connect(pImageViewer,SIGNAL(setZoomOut(bool)),zoomOutAct,SLOT(setEnabled(bool)));

     normalSizeAct = new QAction(tr("&Normal Size"), this);
     normalSizeAct->setEnabled(true);
         normalSizeAct->setIcon(QIcon("icons/zoom_previous.png"));
     connect(normalSizeAct, SIGNAL(triggered()), pImageViewer, SLOT(normalSize()));

     dispAnimationAct = new QAction(tr("&Animation"), this);
     dispAnimationAct->setEnabled(true);
	 dispAnimationAct->setCheckable(true);
         dispAnimationAct->setIcon(QIcon("icons/animation.png"));
     connect(dispAnimationAct, SIGNAL(triggered()), this, SLOT(setDisplayAnimation()));

     dispDifferenceAct = new QAction(tr("&Difference"), this);
     dispDifferenceAct->setEnabled(true);
	 dispDifferenceAct->setCheckable(true);
         dispDifferenceAct->setIcon(QIcon("icons/difference.png"));
     connect(dispDifferenceAct, SIGNAL(triggered()), this, SLOT(setDisplayDifference()));

     dispVxAct = new QAction(tr("V&x"), this);
     dispVxAct->setEnabled(true);
	 dispVxAct->setCheckable(true);
         dispVxAct->setIcon(QIcon("icons/Vx.png"));
     connect(dispVxAct, SIGNAL(triggered()), this, SLOT(setDisplayVx()));

	 dispVyAct = new QAction(tr("V&y"), this);
     dispVyAct->setEnabled(true);
	 dispVyAct->setCheckable(true);
         dispVyAct->setIcon(QIcon("icons/Vy.png"));
     connect(dispVyAct, SIGNAL(triggered()), this, SLOT(setDisplayVy()));

	 toolBar=new QToolBar();
	 toolBar->addAction(zoomInAct);
	 toolBar->addAction(zoomOutAct);
	 toolBar->addAction(normalSizeAct);
	 toolBar->addAction(dispAnimationAct);
	 toolBar->addAction(dispDifferenceAct);
	 toolBar->addAction(dispVxAct);
	 toolBar->addAction(dispVyAct);
}

void FlowFieldViewer::setDisplayAnimation()
{
	pImageViewer->setDisplayType(ImageViewer::Animation);
	pImageViewer->update();

	dispAnimationAct->setChecked(true);
	dispDifferenceAct->setChecked(false);
	dispVxAct->setChecked(false);
	dispVyAct->setChecked(false);
}

void FlowFieldViewer::setDisplayDifference()
{
	pImageViewer->setDisplayType(ImageViewer::Difference);
	pImageViewer->update();

	dispAnimationAct->setChecked(false);
	dispDifferenceAct->setChecked(true);
	dispVxAct->setChecked(false);
	dispVyAct->setChecked(false);
}

void FlowFieldViewer::setDisplayVx()
{
	pImageViewer->setDisplayType(ImageViewer::DisplayVx);
	pImageViewer->update();

	dispAnimationAct->setChecked(false);
	dispDifferenceAct->setChecked(false);
	dispVxAct->setChecked(true);
	dispVyAct->setChecked(false);
}

void FlowFieldViewer::setDisplayVy()
{
	pImageViewer->setDisplayType(ImageViewer::DisplayVy);
	pImageViewer->update();

	dispAnimationAct->setChecked(false);
	dispDifferenceAct->setChecked(false);
	dispVxAct->setChecked(false);
	dispVyAct->setChecked(true);
}


void FlowFieldViewer::loadImage(const QImage &image)
{
	pImageViewer->loadImage(image);
	pImageViewer->update();
}

void FlowFieldViewer::loadImage(const QImage &image1, const QImage &image2)
{
	pImageViewer->loadImage2(image1,image2);
	pImageViewer->update();
}

void FlowFieldViewer::loadImage(const QImage &image1, const QImage &image2,const DImage& vx,const DImage& vy)
{
	//don't load image if generating strokes...
	if (!genStrokes)
	{
		pImageViewer->loadImage2(image1,image2,vx,vy);
		pImageViewer->update();
	}
}
