#ifndef _ImageMatching_h
#define _ImageMatching_h

#include "ImPyramid.h"
#include "Image.h"
#include "SortQueue.h"
#include "BinaryImage.h"
#include "MaskPyramid.h"
#include "Stochastic.h"

class ImageMatching
{
private:
	static int _MRatio;
	static const MaskPyramid* pMaskPyramid;
	static bool IsMaskSpecified;
public:
	ImageMatching(void);
	~ImageMatching(void);
	static void setMask(const MaskPyramid* maskPyramid){pMaskPyramid=maskPyramid;IsMaskSpecified=true;};
	static void clearMask(){pMaskPyramid=NULL;IsMaskSpecified=false;};
	static void setQueueRatio(int ratio){_MRatio=ratio;};
	static void MatchCoarse2Fine(int x1,int y1,double& x2,double& y2,const ImPyramid& ImPyramid1,const ImPyramid& ImPyramid2,int searchsize,int winsize,double sigma=-1);
	static void MatchCoarse2Fine(int x1,int y1,int x2,int y2,SortQueue<Point> & sortqueue,const ImPyramid& ImPyramid1,const ImPyramid& ImPyramid2,int searchsize,int winsize,double sigma=-1);
	
	template <class T>
	static void MatchPoint(int x1,int y1,int x2,int y2,SortQueue<Point>& sortqueue,const BinaryImage& searchArea,const Image<T>& image1,const Image<T>& image2,int searchSize,int winSize,double Sigma,int level);

	static void LucasKanade(int x1,int y1,double& x2,double& y2,const ImPyramid& ImPyramid1,const ImPyramid& ImPyramid2,int winsize,double sigma=-1);
};


template <class T>
void ImageMatching::MatchPoint(int x1, int y1, int x2, int y2, SortQueue<Point> &sortqueue, const BinaryImage &searchArea, const Image<T>& image1,const Image<T>& image2,int searchSize,int winSize,double Sigma,int level)
{
	double* pGaussian;
	pGaussian=new double[(winSize*2+1)*(winSize*2+1)];
	CStochastic::Generate2DGaussian(pGaussian,winSize,Sigma);
	// modulate the Gaussian with a mask
	if(IsMaskSpecified && pMaskPyramid!=NULL && level<pMaskPyramid->nLevels())
	{
		const DImage& mask=pMaskPyramid->mask(level);
		double nMaskPixels=0;
		for(int i=-winSize;i<=winSize;i++)
			for(int j=-winSize;j<=winSize;j++)
			{
				int x,y;
				x=x1+j;
				y=y1+i;
				if(x<0 || y<0 || x>=mask.width() || y>=mask.height())
					continue;
				nMaskPixels+=mask.data()[y*mask.width()+x];
			}
		if(nMaskPixels>winSize/2) // there has to be certain number of mask pixels
		{
			for(int i=-winSize;i<=winSize;i++)
				for(int j=-winSize;j<=winSize;j++)
				{
					int x,y;
					x=x1+j;
					y=y1+i;
					if(x<0 || y<0 || x>=mask.width() || y>=mask.height())
						continue;
					pGaussian[(i+winSize)*(2*winSize+1)+j+winSize]*=mask.data()[y*mask.width()+x];
				}
			DImage temp(winSize*2+1,winSize*2+1);
			memcpy(temp.data(),pGaussian,sizeof(double)*(winSize*2+1)*(winSize*2+1));
			temp.imwrite("mask_"+QString::number(level+1)+".bmp",ImageIO::normalized);
		}
	}

	const T* pData1=image1.data();
	const T* pData2=image2.data();

	double temp,threshold;
	if(image1.IsFloat())
		threshold=0.4;
	else
		threshold=100;

	int nChannels=image1.nchannels();
	int winLength=winSize*2+1;

	for(int i=-searchSize;i<=searchSize;i++)
		for(int j=-searchSize;j<=searchSize;j++)
		{
			if(searchArea.getPixel(j+searchSize,i+searchSize)==0)
				continue;
			if(image2.IsInImage(x2+j,y2+i)==false)
				continue;

			double totalScore=0;
			double totalWeight=0;
			// compute the matching score
			for(int u=-winSize;u<=winSize;u++)
				for(int v=-winSize;v<=winSize;v++)
				{
					int xx1=x1+v;
					int yy1=y1+u;
					if(!image1.IsInImage(xx1,yy1))
						continue;
					int xx2=x2+j+v;
					int yy2=y2+i+u;
					if(!image2.IsInImage(xx2,yy2))
						continue;
					int offset1=(yy1*image1.width()+xx1)*nChannels;
					int offset2=(yy2*image2.width()+xx2)*nChannels;
					double weight=pGaussian[(u+winSize)*winLength+v+winSize];
					for(int k=0;k<nChannels;k++)
					{
						temp=__min(fabs((double)pData1[offset1+k]-pData2[offset2+k]),threshold);
						totalScore+=temp*weight;
						totalWeight+=weight;
					}
				}
			sortqueue.Push(totalScore/totalWeight,Point(x2+j,y2+i));
		}
	//for(int i=0;i<sortqueue.num();i++)
	//	cout<<sortqueue.score(i)<<" ("<<sortqueue[i].x()<<" "<<sortqueue[i].y()<<")"<<endl;
	//cout<<endl;
	delete pGaussian;
}

#endif
