
#ifndef _IMAGEVIEWER_H
#define _IMAGEVIEWER_H

#include <QWidget>
#include <QImage>
#include <QPainter>
#include <QBasicTimer>
#include <QTabletEvent>
#include "Image.h"



class QImage;

//-----------------------------------------------------------------------------
// Image viewer is set to be a generic image viewing widget
//----------------------------------------------------------------------------- 
class ImageViewer :  public QWidget
{
	Q_OBJECT
public:
	// MouseAction may be defined at elsewhere
	enum DisplayType{Animation,Difference,DisplayVx,DisplayVy,DisplayVectorField};
	enum MouseAction{LeftClick,RightClick,Move,DoubleClick,Release};
public:
	ImageViewer(QWidget* parent=0);
	~ImageViewer(void);
	void setParentSize(QSize size);
	inline void setMinScale(double min_scale){minScale=min_scale;};
	inline void setMaxScale(double max_scale){maxScale=max_scale;};
	inline double getScaleFactor(){return scaleFactor;};
	inline QPoint getPaintingOffset(){return QPoint(HScreenOffset, VScreenOffset);};
	inline QPoint getImageSize(){return QPoint(ImWidth, ImHeight);};
	void computeDiffImage();
	void computeWarpImage();
	void computeFlow();
	void setDisplayType(DisplayType type){displayType=type;update();};
protected:
	virtual void paintEvent(QPaintEvent *event);
	virtual void mousePressEvent ( QMouseEvent * event );
	virtual void mouseMoveEvent ( QMouseEvent * event );
	virtual void mouseDoubleClickEvent ( QMouseEvent * event );
	virtual void mouseReleaseEvent ( QMouseEvent * event );
	virtual void timerEvent(QTimerEvent *event);
	//virtual void tabletEvent(QTabletEvent *event);

signals:
	void adjustScrollBar(double factor);
	void setZoomIn(bool flag);
	void setZoomOut(bool flag);
	void imageResized(double scaleFactor);
	void viewerMouseEvent(ImageViewer& imageViewer, ImageViewer::MouseAction& mouseAction,QPoint& pos,float pressure,Qt::KeyboardModifiers& km);
	void paintingImageViewer(ImageViewer& imageViewer,QPainter& painter,QImage& img);
	void paintStrokes(ImageViewer& imageViewer,QPainter& painter);
	//void computeWarpImage2(const QImage& Im2,QImage& warpIm2,double ScaleFactor);
	void finishedPainting();

public slots:
	void loadImage(const QImage& inputImage);
	void loadImage2(const QImage& image1,const QImage& image2);
	void loadImage2(const QImage& image1,const QImage& image2,const DImage& vx,const DImage& vy);
    void zoomIn();
    void zoomOut();
    void normalSize();
	void redraw(){update();};
	void reset();
	void switchDisplay();
	void showImage(){showingImage = true;};
	void hideImage(){showingImage = false;};
	void imageTransparencyChanged(int imageTransparency){_imageTransparency = ((float)imageTransparency)/100.0;update();};
	

private:
	void resizeImage(double factor=0);
	void computeViewerSize();
	int ImHeight,ImWidth;// the dimension of the image
	int HMargin,VMargin;// the horizontal and vertical margin of the viewing on the canvas
	int xoffset,yoffset;// the offset to the top left corner of the frame
	int HScreenOffset,VScreenOffset; // the offset of the top-left corner of the image
	int ViewerHeight,ViewerWidth; // the dimension of the image viewer, ViewerHeight=ImHeight+HMargin*2+2;

	QImage m_Image,dispImage;

	float _imageTransparency;
	
	bool showingImage;

	bool IsDualImage; // if IsDualImage==true, then the image viewer is designed to display two images
	QImage m_Image2,dispImage2;
	DImage Vx,Vy;
	QImage m_Vx,m_Vy;
	QImage dispVx,dispVy;  // members to display the flow field

	double min_Vx,max_Vx,min_Vy,max_Vy;

	QImage m_WarpImage2,m_DiffImage; 
	QImage dispWarpImage2,dispDiffImage;

	DisplayType displayType;
	double scaleFactor;
	double minScale,maxScale;
	QSize parentViewerSize;// the size of the parent widget


	QBasicTimer timer;
	unsigned int steps;

};

#endif