#include "Inspector.h"
#include <QtGui>
#include <QtGui/QApplication>
#include <QApplication>
 #include <QHeaderView>
 #include <QItemSelectionModel>
 #include <QStandardItemModel>
#include <iostream>
#include "ParameterDelegate.h"
#include "Correspondence.h"

using namespace std;

Inspector::Inspector(QWidget *parent, Qt::WFlags flags)
	: QWidget(parent, flags)
{
	IsAddPara=false;
	IsActive=false;
	IsSettingValue=false;

	//-------------------------------------------------------------------------------------
	// set optical flow parameters
	//-------------------------------------------------------------------------------------
	// take care of the QTableWidget
	tableWidget=new QTableWidget(0,5);
	tableWidget->setItemDelegate(new ParameterDelegate);	

	tableWidget->setColumnWidth(0,20);
	tableWidget->setColumnWidth(1,60);
	tableWidget->setColumnWidth(2,60);
	tableWidget->setColumnWidth(3,45);
	tableWidget->setColumnWidth(4,80);

	QStringList headerLabels;
     headerLabels << "" << "Alpha" << "Gamma"<<"Eta"<<"Motion type" ;
     tableWidget->setHorizontalHeaderLabels(headerLabels);
	
	 //motionPara.addFlowPara(OpticalFlowPara(1,0.05,0.02));
	 //motionPara.addFlowPara(OpticalFlowPara(1,0.02,0.01));
	 //motionPara.addFlowPara(OpticalFlowPara(1,0.01,0.002));
	 //for(int i=0;i<motionPara.numFlowPara();i++)
		// addOpticalFlowPara(motionPara.flowPara(i));

	 tableWidget->setFixedWidth(300);
	 tableWidget->setMaximumHeight(240);

	 connect(tableWidget,SIGNAL(cellClicked(int,int)),this,SLOT(selectCell(int,int)));
	 connect(tableWidget,SIGNAL(cellChanged(int,int)),this,SLOT(modifyData(int,int)));
	 // the push buttons
         addParaButton=new QPushButton(QIcon("icons/list-add.png"),"");
	 addParaButton->setMaximumWidth(22);
	 connect(addParaButton,SIGNAL(clicked()),this,SLOT(addOpticalFlowParaSet()));
         removeParaButton=new QPushButton(QIcon("icons/list-remove.png"),"");
	 removeParaButton->setMaximumWidth(22);
	 connect(removeParaButton,SIGNAL(clicked()),this,SLOT(removeOpticalFlowParaSet()));

	 // the layout of the buttons
	 buttonLayout=new QHBoxLayout;
	 buttonLayout->addWidget(addParaButton);
	 buttonLayout->addWidget(removeParaButton);
	 buttonLayout->addSpacing(180);

	 // the layout of the other parameters
	 ratioLabel=new QLabel("Down sampling ratio (0.50~0.95): ");
	 minWidthLabel=new QLabel("Min width at top level (5~30):");
	 nIterationsLabel=new QLabel("Number of iterations (5~40):");
	 nIRLSIterationsLabel=new QLabel("Number of IRLS iterations (1~4):");
	 nCGIterationsLabel=new QLabel("Number of CG iterations (10~60):");

	 ratioSpinBox=new QDoubleSpinBox;
	 ratioSpinBox->setRange(0.5,0.95);
	 ratioSpinBox->setDecimals(2);
	 ratioSpinBox->setSingleStep(0.01);
	 ratioSpinBox->setMaximumWidth(50);
	 ratioSpinBox->setEnabled(false);
	 connect(ratioSpinBox,SIGNAL(valueChanged(double)),this,SLOT(setRatio(double)));

	 minWidthSpinBox=new QSpinBox;
	 minWidthSpinBox->setRange(5,30);
	 minWidthSpinBox->setMaximumWidth(50);
	 minWidthSpinBox->setEnabled(false);
	 connect(minWidthSpinBox,SIGNAL(valueChanged(int)),this,SLOT(setMinWidth(int)));

	 nIterationsSpinBox=new QSpinBox;
	 nIterationsSpinBox->setRange(5,40);
	 nIterationsSpinBox->setMaximumWidth(50);
	 nIterationsSpinBox->setEnabled(false);
	 connect(nIterationsSpinBox,SIGNAL(valueChanged(int)),this,SLOT(setnIterations(int)));

	 nIRLSIterationsSpinBox=new QSpinBox;
	 nIRLSIterationsSpinBox->setRange(1,4);
	 nIRLSIterationsSpinBox->setMaximumWidth(50);
	 nIRLSIterationsSpinBox->setEnabled(false);
	 connect(nIRLSIterationsSpinBox,SIGNAL(valueChanged(int)),this,SLOT(setnIRLSIterations(int)));

	 nCGIterationsSpinBox=new QSpinBox;
	 nCGIterationsSpinBox->setRange(10,60);
	 nCGIterationsSpinBox->setMaximumWidth(50);
	 nCGIterationsSpinBox->setEnabled(false);
	 connect(nCGIterationsSpinBox,SIGNAL(valueChanged(int)),this,SLOT(setnCGIterations(int)));
	
	 // set the parameters 
	 setEditing();

	 ratioLayout=new QHBoxLayout;
	 ratioLayout->addWidget(ratioLabel);
	 ratioLayout->addWidget(ratioSpinBox);
	
	 minWidthLayout=new QHBoxLayout;
	 minWidthLayout->addWidget(minWidthLabel);
	 minWidthLayout->addWidget(minWidthSpinBox);

	 nIterationsLayout=new QHBoxLayout;
	 nIterationsLayout->addWidget(nIterationsLabel);
	 nIterationsLayout->addWidget(nIterationsSpinBox);

	 nIRLSIterationsLayout=new QHBoxLayout;
	 nIRLSIterationsLayout->addWidget(nIRLSIterationsLabel);
	 nIRLSIterationsLayout->addWidget(nIRLSIterationsSpinBox);

	 nCGIterationsLayout=new QHBoxLayout;
	 nCGIterationsLayout->addWidget(nCGIterationsLabel);
	 nCGIterationsLayout->addWidget(nCGIterationsSpinBox);

	 // the layout of the optical flow tab
	 vLayout=new QVBoxLayout;
	 vLayout->addWidget(tableWidget);
	 vLayout->addLayout(buttonLayout);
	 vLayout->addLayout(ratioLayout);
	 vLayout->addLayout(minWidthLayout);
	 vLayout->addLayout(nIterationsLayout);
	 vLayout->addLayout(nIRLSIterationsLayout);
	 vLayout->addLayout(nCGIterationsLayout);

	 OpticalFlow=new QWidget;
	 OpticalFlow->setLayout(vLayout);

	//-------------------------------------------------------------------------------------
	// set manual parameters
	//-------------------------------------------------------------------------------------
	m_SearchSize=20;
	m_WinSize=15;
	IsUncertaintyMatching=false;

	 spinBoxSearchSize=new QSpinBox;
	 sliderSearchSize=new QSlider(Qt::Horizontal);
	 spinBoxSearchSize->setRange(5,45);
	 sliderSearchSize->setRange(5,45);
	 sliderSearchSize->setFixedWidth(150);
	 connect(spinBoxSearchSize,SIGNAL(valueChanged(int)),sliderSearchSize,SLOT(setValue(int)));
	 connect(sliderSearchSize,SIGNAL(valueChanged(int)),spinBoxSearchSize,SLOT(setValue(int)));
	 connect(sliderSearchSize,SIGNAL(valueChanged(int)),this,SLOT(setSearchSizeFromSlider(int)));
	 sliderSearchSize->setValue(m_SearchSize);

	 spinBoxWinSize=new QSpinBox;
	 sliderWinSize=new QSlider(Qt::Horizontal);
	 spinBoxWinSize->setRange(5,25);
	 sliderWinSize->setRange(5,25);
	 sliderWinSize->setFixedWidth(150);
	 connect(spinBoxWinSize,SIGNAL(valueChanged(int)),sliderWinSize,SLOT(setValue(int)));
	 connect(sliderWinSize,SIGNAL(valueChanged(int)),spinBoxWinSize,SLOT(setValue(int)));
	 connect(sliderWinSize,SIGNAL(valueChanged(int)),this,SLOT(setWinSizeFromSlider(int)));
	 sliderWinSize->setValue(m_WinSize);

	 manualMatching=new QCheckBox("Manual matching (increase resolution first)");
	 manualMatching->setChecked(false);
	 connect(manualMatching,SIGNAL(clicked(bool)),this,SIGNAL(manualMatchingChanged(bool)));

	 enableUncertaintyMatching=new QCheckBox("Enable uncertainty matching");
	 enableUncertaintyMatching->setChecked(false);
	 connect(enableUncertaintyMatching,SIGNAL(clicked(bool)),this,SIGNAL(uncertainMatchingChanged(bool)));

	 //QButtonGroup* motionSelection=new QButtonGroup;
	 button1=new QRadioButton("Translation");
	 button2=new QRadioButton("Affine motion");
	 button3=new QRadioButton("Homography (perspective motion)");
	 button4=new QRadioButton("Piecewise smooth flow field (caution: slow!)");
	 connect(button1,SIGNAL(clicked(bool)),this,SLOT(translationChecked(bool)));
	 connect(button2,SIGNAL(clicked(bool)),this,SLOT(affineChecked(bool)));
	 connect(button3,SIGNAL(clicked(bool)),this,SLOT(homographyChecked(bool)));
	 connect(button4,SIGNAL(clicked(bool)),this,SLOT(piecewisesmoothChecked(bool)));
	 button1->setChecked(true);

	 lambdaSpinBox=new QDoubleSpinBox();
	 lambdaSpinBox->setMinimum(0.01);
	 lambdaSpinBox->setMaximum(100);
	 lambdaSpinBox->setValue(1);
	 lambdaSpinBox->setSingleStep(0.01);
	 lambdaSpinBox->setMaximumWidth(60);
	 connect(lambdaSpinBox,SIGNAL(valueChanged(double)),this,SIGNAL(lambdaChanged(double)));


	 //motionSelection->addButton(button1);
	 //motionSelection->addButton(button2);
	 //motionSelection->addButton(button3);
	 //motionSelection->addButton(button4);
	 //motionSelection->setExclusive(true);
	 
	 QGroupBox *groupBox=new QGroupBox("Select motion type:");
	 QVBoxLayout* vbox=new QVBoxLayout;
	 vbox->addWidget(button1);
	 vbox->addWidget(button2);
	 vbox->addWidget(button3);
	 vbox->addWidget(button4);
	 groupBox->setLayout(vbox);

	 clearLabelingButton=new QPushButton("Clear labeling");
	 clearLabelingButton->setFixedWidth(100);
	 connect(clearLabelingButton,SIGNAL(pressed()),SIGNAL(clearLabeling()));

	 redoMatchingButton=new QPushButton("Redo matching");
	 redoMatchingButton->setFixedWidth(100);
	 connect(redoMatchingButton,SIGNAL(pressed()),SIGNAL(redoMatching()));

	 grabFeaturesButton=new QPushButton("Feature from previous frame");
	 grabFeaturesButton->setFixedWidth(180);
	 connect(grabFeaturesButton,SIGNAL(pressed()),SIGNAL(grabFeatures()));

	 vLayout=new QVBoxLayout;
	
	 hLayout=new QHBoxLayout;
	 hLayout->addWidget(new QLabel("Search window size:"));
	 hLayout->addWidget(spinBoxSearchSize);
	 hLayout->addWidget(sliderSearchSize);
	 vLayout->addLayout(hLayout);
	 hLayout=new QHBoxLayout;
	 hLayout->addWidget(new QLabel("Patch window size:"));
	 hLayout->addWidget(spinBoxWinSize);
	 hLayout->addWidget(sliderWinSize);
	 vLayout->addLayout(hLayout);
	 vLayout->addWidget(manualMatching);
	 vLayout->addWidget(enableUncertaintyMatching);
	 vLayout->addWidget(groupBox);
	 hLayout=new QHBoxLayout;
	 hLayout->addWidget(new QLabel("Lambda (the regularization for flow interpolation): "));
	 hLayout->addWidget(lambdaSpinBox);
	 vLayout->addLayout(hLayout);
	 hLayout=new QHBoxLayout;
	 hLayout->addWidget(clearLabelingButton);
	 hLayout->addWidget(redoMatchingButton);
	 vLayout->addLayout(hLayout);
	 hLayout=new QHBoxLayout;
	 //hLayout->addStretch(1);
	 hLayout->addWidget(grabFeaturesButton);
	 vLayout->addLayout(hLayout);
	 vLayout->addStretch(1);

	 Manual=new QWidget;
	 Manual->setLayout(vLayout);

	//-------------------------------------------------------------------------------------
	// set tabs
	//-------------------------------------------------------------------------------------
	 tabGroup=new QTabWidget;
	 
	 Parametric=new QWidget;
	
	 tabGroup->addTab(OpticalFlow,"Optical flow");
	 tabGroup->addTab(Parametric,"Parametric");
	 tabGroup->addTab(Manual,"Manual");
	 connect(tabGroup,SIGNAL(currentChanged(int)),this,SIGNAL(tabChanged(int)));

	vGlobalLayout=new QVBoxLayout;
	vGlobalLayout->addWidget(tabGroup);
	setLayout(vGlobalLayout);

	 setMaximumWidth(330);
         setWindowIcon(QIcon("icons/preferences-system.png"));
	 setWindowTitle("Inspector");

	 timer.start(500,this);
	steps=0;
}

Inspector::~Inspector()
{

}

void Inspector::setLambda(double lambda)
{
	lambdaSpinBox->setValue(lambda);
}

void Inspector::selectTab(int index)
{
	tabGroup->setCurrentIndex(index);
}

void Inspector::setEditing()
{
	ratioSpinBox->setValue(motionPara.ratio());
	minWidthSpinBox->setValue(motionPara.minWidth());
	nIterationsSpinBox->setValue(motionPara.nIterations());
	nIRLSIterationsSpinBox->setValue(motionPara.nIRLSIterations());
	nCGIterationsSpinBox->setValue(motionPara.nCGIterations());
}

void Inspector::addOpticalFlowPara(const OpticalFlowPara &para)
{
	IsAddPara=true;
	tableWidget->insertRow(tableWidget->rowCount());
	int row=tableWidget->rowCount()-1;
	for (int column = 0; column < 5; ++column) 
	{
		 QTableWidgetItem *item=new QTableWidgetItem;
		 if(column==0)
			 item->setData(0,QVariant((int)para.IsActive()));
		 if(column==1)
			  item->setData(0,QVariant(para.alpha()));
		 if(column==2)
			 item->setData(0,QVariant(para.gamma()));
		 if(column==3)
			 item->setData(0,QVariant(para.eta()));
		 if(column==4)
		 {
			 if(para.regularization()==OpticalFlowPara::membrane)
				item->setData(0,QVariant(QString("membrane")));
			 else
				 item->setData(0,QVariant(QString("thinplate")));
		 }
		 item->setTextAlignment(Qt::AlignHCenter+Qt::AlignVCenter);
		 tableWidget->setItem(row,column,item);
	}
	QTableWidgetItem* widgetItem=new  QTableWidgetItem(QString::number(row+1));
	tableWidget->setVerticalHeaderItem(row,widgetItem);

	tableWidget->setRowHeight(row,17);
	IsAddPara=false;
}

void Inspector::addOpticalFlowParaSet()
{
	if(IsActive==false)
		return;
	motionPara.addFlowPara(OpticalFlowPara());
	motionPara.dispFlowPara();
	addOpticalFlowPara(motionPara.flowParaList().last());
	tableWidget->update();
	emit paraModified(motionPara);
}

void Inspector::removeOpticalFlowParaSet()
{
	if(IsActive==false)
		return;

	if(motionPara.flowParaList().isEmpty())
		return;
	int column=tableWidget->currentColumn();
	int row=tableWidget->currentRow();
	if(row<0 || row>=tableWidget->rowCount())
		return;

	for(int i=0;i<tableWidget->columnCount();i++)
		if(tableWidget->item(row,i)->isSelected()==false)
			return;

	tableWidget->removeRow(row);
	motionPara.removeFlowPara(row);
	for(int i=row;i<tableWidget->rowCount();i++)
		tableWidget->verticalHeaderItem(i)->setText(QString::number(i+1));
	emit paraModified(motionPara);
}

void Inspector::selectCell(int row, int column)
{
	if(IsActive==false)
		return;

	//tableWidget->selectRow(row);
	//printf("%d %d\n",row,column);
	emit selectedRow(row);
}

void Inspector::selectRow(int row)
{
	if(row>=0 && row<tableWidget->rowCount())
		tableWidget->selectRow(row);
}

void Inspector::timerEvent(QTimerEvent *event)
 {
     if (event->timerId() == timer.timerId()) {
         ++steps;
		 if(tableWidget->rowCount()==0)
			 return;
		 for(int row=0;row<tableWidget->rowCount();row++)
		 {
			 bool isRowSelected=true;
			 for(int i=0;i<tableWidget->columnCount();i++)
				 if(tableWidget->item(row,i)->isSelected()==false)
				 {
					 isRowSelected=false;
					 break;
				 }

			 if(isRowSelected)
			 {
				emit selectedRow(row);
				break;
			}
		 }
     } else {
         QWidget::timerEvent(event);
	 }
} 


void Inspector::modifyData(int row,int column)
{
	if(IsActive==false)
		return;
	if(IsAddPara)
		return;
	//printf("nelements: %d\n",motionPara.numFlowPara());
	switch(column){
		case 0:
			motionPara.flowParaList()[row].IsActive()=tableWidget->item(row,column)->data(Qt::DisplayRole).toBool();
			break;
		case 1:
			motionPara.flowParaList()[row].alpha()=tableWidget->item(row,column)->data(Qt::DisplayRole).toDouble();
			break;
		case 2:
			motionPara.flowParaList()[row].gamma()=tableWidget->item(row,column)->data(Qt::DisplayRole).toDouble();
			break;
		case 3:
			motionPara.flowParaList()[row].eta()=tableWidget->item(row,column)->data(Qt::DisplayRole).toDouble();
		case 4:
			if(tableWidget->item(row,column)->data(Qt::DisplayRole).toString()=="membrane")
				motionPara.flowParaList()[row].regularization()=OpticalFlowPara::membrane;
			else
				motionPara.flowParaList()[row].regularization()=OpticalFlowPara::thinplate;
			break;
	}
	emit paraModified(motionPara);
}

void Inspector::paraLoaded(const MotionPara& mPara)
{
	//mPara.dispFlowPara();
	IsActive=true;
	ratioSpinBox->setEnabled(true);
	minWidthSpinBox->setEnabled(true);
	nIterationsSpinBox->setEnabled(true);
	nIRLSIterationsSpinBox->setEnabled(true);
	nCGIterationsSpinBox->setEnabled(true);

	// clear the table view
	for(int i=0;i<motionPara.numFlowPara();i++)
		tableWidget->removeRow(0);

	// copy the parameters
	motionPara=mPara;

	 // set the parameters 
	 setEditing();

	 for(int i=0;i<mPara.numFlowPara();i++)
		 addOpticalFlowPara(mPara.flowParaList()[i]);
	 
	tableWidget->update();
}

void Inspector::clearPara()
{
	IsActive=false;
	for(int i=0;i<motionPara.numFlowPara();i++)
		tableWidget->removeRow(0);
	motionPara.flowParaList().clear();
	tableWidget->update();
	ratioSpinBox->setEnabled(false);
	minWidthSpinBox->setEnabled(false);
	nIterationsSpinBox->setEnabled(false);
	nIRLSIterationsSpinBox->setEnabled(false);
	nCGIterationsSpinBox->setEnabled(false);
}

void Inspector::setRatio(double _ratio)
{
	motionPara.ratio()=_ratio;
	if(IsActive)
		emit paraModified(motionPara);
}

void Inspector::setMinWidth(int _minWidth)
{
	motionPara.minWidth()=_minWidth;
	if(IsActive)
		emit paraModified(motionPara);
}

void Inspector::setnIterations(int _nIterations)
{
	motionPara.nIterations()=_nIterations;
	if(IsActive)
		emit paraModified(motionPara);
}

void Inspector::setnIRLSIterations(int _nIRLSIterations)
{
	motionPara.nIRLSIterations()=_nIRLSIterations;
	if(IsActive)
		emit paraModified(motionPara);
}

void Inspector::setnCGIterations(int _nCGIterations)
{
	motionPara.nCGIterations()=_nCGIterations;
	if(IsActive)
		emit paraModified(motionPara);
}

//------------------------------------------------------------------------------------------
// functions related to manual tab
//------------------------------------------------------------------------------------------
void Inspector::setSearchSizeFromSlider(int searchSize)
{
	if(IsSettingValue)
		return;
	m_SearchSize=searchSize;
	emit searchSizeChanged(m_SearchSize);
}

void Inspector::setWinSizeFromSlider(int winSize)
{
	if(IsSettingValue)
		return;
	m_WinSize=winSize;
	emit winSizeChanged(m_WinSize);
}

void Inspector::setSearchingWindowSize(int searchSize,int winSize)
{
	m_SearchSize=searchSize;
	m_WinSize=winSize;
	IsSettingValue=true;
	sliderSearchSize->setValue(searchSize);
	sliderWinSize->setValue(winSize);
	IsSettingValue=false;
}

void Inspector::translationChecked(bool checked)
{
	if(checked)
	{
		cout<<"Translation"<<endl;
		emit motionTypeSelected(0);
	}
}

void Inspector::affineChecked(bool checked)
{
	if(checked)
	{
		cout<<"Affine"<<endl;
		emit motionTypeSelected(1);
	}
}

void Inspector::homographyChecked(bool checked)
{
	if(checked)
	{
		cout<<"Homography"<<endl;
		emit motionTypeSelected(2);
	}
}

void Inspector::piecewisesmoothChecked(bool checked)
{
	if(checked)
	{
		cout<<"Piecewise smooth"<<endl;
		emit motionTypeSelected(3);
	}
}

void Inspector::setMotionType(int type)
{
	button1->setChecked(type==0);
	button2->setChecked(type==1);
	button3->setChecked(type==2);
	button4->setChecked(type==3);
}
