#ifndef _LayerObject_h
#define _LayerObject_h

#include <QObject>
#include <QSet>
#include <QList>
#include "ContourLabel.h"
#include "LayerDepth.h"
#include "XmlWriter.h"
#include "BinaryImage.h"
#include "Inspector.h"
#include "Motion.h"
#include "Image.h"
#include "time.h"
#include "Correspondence.h"
#include <QProgressDialog>
#include "SortQueue.h"
#include "Stroke.h"
#include "StrokeProperties.h"
#include "StrokeLayerProperties.h"
#include "ANN/ANN.h"
#include "jl_vectors.h"

class ContourLabel;
class LayerDepth;
class XmlWriter;

#define FREE_TEXTURE -1
#define NO_IMAGE 0
#define TEXTURE_EXISTS 1
#define CREATE_TEXTURE 2



class LayerObject: public QObject
{
	Q_OBJECT

		// the member variables
private:
	enum Selected{Nothing,BoundaryContour,StrokeContour};

	QList<ContourLabel> m_ContourLabelList;			// the contour labels, from the first frame to the last 
	QList<double> m_DepthList;												// the depth, as a variable of frame index
	QString m_LayerName;

	QList<bool> m_IsGlobalLabelList;
	QList<bool> m_IsLocalLabelList;
	QList<bool> m_IsTrackedList;
	QList<bool> m_IsDepthLabeledList;
	QList<int> m_TabIndexList;

	int m_NumFrames;   // the total number of frames, accessed by numFrames();
	int m_StartFrame,m_EndFrame; // the start and end frame index, range [0, numFrames()-1]. accessed by startFrame() and endFrame();
	int m_CreatedFrame; // the index of the frame where the object is created;

	static int selectedPointIndex;
	static ContourLabel m_OrgContourLabel;
	static QPointF m_OrgCursorPoint;

	// the list for the layer mask
	QList<BinaryImage> m_LayerMaskList;
	QList<BinaryImage> m_VisibleLayerMaskList;
	QList<QRect> m_BoundingBoxList;
	QList<QRect> m_FullBoundingBoxList;
	QList<IntegerImage> m_BoundaryDistanceImageList;
	QList<IntegerImage> m_BoundaryDistanceDxImageList;
	QList<IntegerImage> m_BoundaryDistanceDyImageList;
	QList<bool> m_IsBoundaryDistanceCreatedList;

	MotionPara m_LayerMotionPara;    // the list for dense optical flow fields
	QList<Motion> m_LayerMotionList;
	QList<Motion> m_LayerReverseMotionList;
	int m_SelectedRow;  // the row where the current viewer is viewing

	QList<Correspondence> m_CorrespondenceList; // the list for sparse correspondence

	int m_CurrentStroke;
    QList<QList<ContourLabel> > m_StrokeList;
	int m_Selected;
	QList<ContourLabel> m_StrokeListCopy;

    QList<QMap<int,QVector<Stroke *> > > m_GeneratedStrokeList;
	QMap<int,StrokeLayerProperties > m_GeneratedStrokeParameterList;

	QList<QImage> m_StrokeImageList;
	QList<int> m_StrokeImageTextureList;
	QList<int> m_StrokeImageStateList;


	QList<ANNkd_tree*>m_GuideStrokeKDTreeList;
	QList<ANNkd_tree*>m_ColorStrokeKDTreeList;
	QList<ANNkd_tree*>m_BoundaryKDTreeList;

	int m_StrokeBorderSize;
	float m_StrokeBoundaryBlur;
	int m_StrokeMedianBlur;

	//user strokes motion fields

	//layer boundary motion fields
	QList<FlowField> m_BoundaryFlowList;
	QList<FlowField> m_BoundaryReverseFlowList;
	QList<bool> m_IsBoundaryFlowCreatedList;

	int m_ImageHeight, m_ImageWidth;

	int m_MaxStrokeId;

	QString m_ProjectDir;

    QMap<unsigned int, QList<QList<ContourLabel> > > m_PreviousStrokeLists;

	int m_NearestStroke;

	char _strbuf[2048];

	//int m_TabIndex; // tabIndex indicates the type of motion that will be finally used to compose the optical flow field
public:


	// construction, desctrution, copy, and clear
	LayerObject(void);
	LayerObject(const LayerObject& layerObject);
	~LayerObject(void);
	LayerObject& operator=(const LayerObject& layerObject);
	void copyData(const LayerObject& layerObject);

	QString projectDir(){return m_ProjectDir;};
	void setProjectDir(QString projectDir){m_ProjectDir=projectDir;};
	// functions to directly access the members
	static int SelectedPointIndex() {return selectedPointIndex;};
	inline int numFrames() const{return m_NumFrames;};
	inline int startFrame() const{return m_StartFrame;};
	inline int endFrame() const{return m_EndFrame;};
	inline int createdFrame() const{return m_CreatedFrame;};
	inline QString& layerName()  {return m_LayerName;};
	inline int depth(int frameIndex) const {return m_DepthList[frameIndex];};
	inline ContourLabel& contourLabel(int frameIndex)  {return m_ContourLabelList[frameIndex];};
	inline bool isPolygon(int frameIndex) const {return m_ContourLabelList[frameIndex].isPolygon();};
	inline const BinaryImage& Mask(int frameIndex) const{return m_LayerMaskList[frameIndex];};
	inline const BinaryImage& visibleMask(int frameIndex) const{return m_VisibleLayerMaskList[frameIndex];};
	inline const QRect& boundingBox(int frameIndex) const{return m_FullBoundingBoxList[frameIndex];};// m_BoundingBoxList[frameIndex];
	inline const QRect& fullBoundingBox(int frameIndex) const{return m_FullBoundingBoxList[frameIndex];};//m_FullBoundingBoxList[frameIndex];
	inline const MotionPara& LayerMotionPara() const {return m_LayerMotionPara;};
	inline bool flowExists(int frame) {return ((frame>=0) && (m_LayerMotionList.size() >frame) && (m_LayerMotionList[frame].flowFieldList().size() > m_SelectedRow) && (m_SelectedRow >=0));};
	inline bool reverseFlowExists(int frame) {return ((frame>=0) && (m_LayerReverseMotionList.size() >frame)&& (m_LayerReverseMotionList[frame].flowFieldList().size() > m_SelectedRow) && (m_SelectedRow >=0));};
	inline MotionPara& LayerMotionPara() {return m_LayerMotionPara;};
	inline int selectedRow() const {return m_SelectedRow;};
	inline int& selectedRow() {return m_SelectedRow;};
	inline int tabIndex(int frameIndex) const{return m_TabIndexList[frameIndex];};
	inline int& tabIndex(int frameIndex) {return m_TabIndexList[frameIndex];};
	inline double lambda(int frameIndex) const {return m_CorrespondenceList[frameIndex].lambda();};
	inline double& lambda(int frameIndex) {return m_CorrespondenceList[frameIndex].lambda();};
	inline int currentStroke(){return m_CurrentStroke;};
	inline const QList<QPointF>& getContourPoints(int frameIndex){return m_ContourLabelList[frameIndex].PointList();};
	bool boundaryImageCreated(int frameIndex);
	inline IntegerImage& boundaryDistanceImage(int frameIndex){return m_BoundaryDistanceImageList[frameIndex];};
	inline IntegerImage& boundaryDistanceDxImage(int frameIndex){return m_BoundaryDistanceDxImageList[frameIndex];};
	inline IntegerImage& boundaryDistanceDyImage(int frameIndex){return m_BoundaryDistanceDyImageList[frameIndex];};

	
	inline QList<ContourLabel>& getStrokes(int frameIndex){return m_StrokeList[frameIndex];};
	QList<Stroke *> getVisibleStrokes(int frameIndex,bool guide=true,bool color=true);
	QList<Stroke *> getColorStrokes(int frameIndex);
	QList<Stroke *> getGuideStrokes(int frameIndex);

	bool getBoundaryFlow(bool forward, int frameIndex, float x, float y, float *flowx, float *flowy);
	bool getUserStrokeFlow(bool forward, int frameIndex, float x, float y, QVector<Vec2f>& curPts,QVector<Vec2f>& disp, float *flowx, float *flowy);

	FlowField& layerFlow(int frame);
	FlowField& layerReverseFlow(int frame);

	// function to control the LayerObject
	inline void setLayerName(const QString& name){m_LayerName=name;};
	void create(int numFrames,int frameIndex,const QPointF& point,int layer_index);
	void create(int numFrames);
	void createBackground(int numFrames,int width,int height);
	
	inline void setStartFrame(int startFrame){m_StartFrame=startFrame;};
	inline void setEndFrame(int endFrame){m_EndFrame=endFrame;};
	inline void setCreatedFrame(int createdFrame){m_CreatedFrame=createdFrame;};
	inline void setDepth(int frameIndex,double depth){m_DepthList[frameIndex]=depth;};
	inline void setGlobalLabel(int frameIndex,bool label){m_IsGlobalLabelList[frameIndex]=label;};
	inline void setLocalLabel(int frameIndex,bool label){m_IsLocalLabelList[frameIndex]=label;};
	inline void setTracked(int frameIndex,bool label){m_IsTrackedList[frameIndex]=label;};
	inline void setDepthLabel(int frameIndex,bool label){m_IsDepthLabeledList[frameIndex]=label;};
	inline void setContourLabel(int frameIndex,const ContourLabel& contourLabel){m_ContourLabelList[frameIndex]=contourLabel;};
	inline void setImageSize(int width, int height){m_ImageHeight = height;m_ImageWidth = width;};

	inline bool InRange(int frameIndex){if(frameIndex>=m_StartFrame && frameIndex<=m_EndFrame) return true;else return false;};
	void StretchToFrame(int frameIndex);

	bool ReplicateStrokesToFrame(int curFrameIndex, int destFrameIndex,int actionId,QList<QImage>& frameList,double colorUpdate, float optFlowFactor, float boundFlowFactor, float userFlowFactor, float fixedFlowFactor, float fixedFlowAngle, float fixedFlowDist, float followStrokeFactor,float followFlowFactor);
	void updateSelectedStrokeColor(int curFrameIndex, QList<QImage>& frameList,double colorUpdate,int strokeId=-1);

	// functions severing as a filter to access each ContourLabel at a specified frame
	void DrawCorrespondence(int frameIndex,QPainter& painter,QPointF& ScreenOffset,double ScaleFactor);
	void DrawContour(int frameIndex,QPainter& painter,QPointF& ScreenOffset,double ScaleFactor,bool IsActive=false,bool IsCreating=false,bool IsCursorOver=false,bool boundaryContour=true);
	void DrawStrokes(int frameIndex,QPainter& painter,QPointF& ScreenOffset,double ScaleFactor,bool highlightNearest=false,bool IsActive=false,bool IsCreating=false,bool IsCursorOver=false,bool boundaryContour=true);
	void ShowText(int frameIndex,QPainter& painter,QPointF& ScreenOffset,double ScaleFactor);
	double distanceToContour(int frameIndex,const QPointF& point);
	void addPoint(int frameIndex,const QPointF& point,bool IsLabeled=true);
	void updateLastPoint(int frameIndex,const QPointF& point,bool IsLabeled=true);
	void removeLastPoint(int frameIndex);
	double minPointDistance(int frameIndex,const QPointF& point,bool boundaryContour);
	double minPointDistanceSelectedStrokes(int frameIndex,const QPointF& point);
	double minLineSegDistance(int frameIndex,const QPointF& point,bool boundaryContour);
	bool IsCentroidSelected(int frameIndex,const QPointF& point,double ScaleFactor);
	int selectBoundingBox(int frameIndex,const QPointF& point,double ScaleFactor,bool boundaryContour);
	bool IsStrokeSelected(int frameIndex,const QPointF& point,double ScaleFactor);
	void strokeClicked(int frameIndex);

	// functions that propage the operation of a particular frame to the rest of the frames
	int findFirstLocalLabeledFrame(int frameIndex,int pointIndex,bool& IsForwardLabeld);
	int findLastLocalLabeledFrame(int frameIndex,int pointIndex,bool& IsBackwardLabeld);
	int findContourFirstLocalLabeledFrame(ContourLabel& contour,int frameIndex,int pointIndex,bool& IsForwardLabeld);
	int findContourLastLocalLabeledFrame(ContourLabel& contour,int frameIndex,int pointIndex,bool& IsForwardLabeld);
	int findFirstGlobalLabeledFrame(int frameIndex,bool& IsForwardLabeld);
	int findLastGlobalLabeledFrame(int frameIndex,bool& IsBackwardLabeld);
	void insertPoint(int frameIndex,const QPointF& point,bool boundaryContour);
	void updatePoint(int frameIndex,const QPointF& point,bool boundaryContour);
	void propagateUpdatePoint(int frameIndex,bool boundaryContour);
	void propagateUpdatePoint(int frameIndex,int lastFrame,bool IsForwardLabeled,int firstFrame,bool IsBackwardLabeled);
	void propagateContourUpdatePoint(ContourLabel& contour,int frameIndex,int pointIndex, int lastFrame,bool IsForwardLabeled,int firstFrame,bool IsBackwardLabeled);
	void propagateUpdatePointForward(int frameIndex,int lastFrame);
	void propagateUpdatePointBackward(int frameIndex,int firstFrame);

	void moveContour(int frameIndex,const QPointF& point,bool boundaryContour);
	void propagateMoveContour(int frameIndex,bool boundaryContour,bool selectedMultiple,QPointF shift, int actionId);

	void scaleContour(int frameIndex,const QPointF& point,int selectedBoundingBoxCorner,bool boundaryContour);
	void propagateScaleContour(int frameIndex);

	void removePoint(int curFrameIndex,bool boundaryContour=true);
	void removeLabel(int frameIndex);

	// form LayerDepth;
	void FormLayerDepth(int frameIndex,LayerDepth& layerDepth);
	void SetLayerDepth(const LayerDepth& layerDepth);
	void printLayerDepth();

	// file IO
	void writeXML(XmlWriter& xw,int id=0);

	// function to cut the labeling
	void cutLabeling(int frameIndex);
	void cutForward(int frameIndex);
	void cutBackward(int frameIndex);
	void removeAllTracking();
	void enforceKeyFrame(int frameIndex, bool boundaryContour);
	void removeUserLabels(int frameIndex);
	void extendDepth();
	// track to frame
	bool IsForwardTrackable(int frameIndex);
	bool IsBackwardTrackable(int frameIndex);
	void TrackToFrame(int frameIndex,const QList<QPointF>& contour);

	// functions for creating layer mask
	void CreateMask(int frameIndex,int width,int height);
	void CreateVisibleMask(int frameIndex,const BinaryImage& OMask); // this function must be called after CreateMask()
	bool IsWithinLayer(int frameIndex,const QPointF& point, bool visibleMask=false);

	void CreateStrokeImage(int frameIndex);
	void CreateBoundaryDistanceImage(int frameIndex,int width,int height);
	void clearBoundaryDistanceImages();
	void CreateUserStrokeFlow(int frameIndex);

	// functions for motion analysis
	bool IsLayerVisible(int frameIndex);
	void clearMotionAnalysis();
	void MotionAnalysis(int frameIndex,const DImage& image1,const DImage& image2,int &total,QProgressDialog* progressDialog=NULL,time_t* start=NULL,int sum=0);
	void ComposeMotion(int frameIndex,DImage& vx,DImage& Dy,const BiImage& refImage);
	void OutputLayerMotion(int frameIndex,const QString& filename);
	// functions for FILE IO
	bool writeLayerObject(QFile& file) const;
	bool readLayerObject(QFile& file,int nFrames,QString& projectDir);
	// function to reduce frame range
	bool reduceFrameRange(int start,int end);
	void removeFrames(int frameIndex, bool forward);

	bool getSelectedMotion(int frameIndex,int row,QImage& patch1,QImage& patch2,DImage& vx,DImage& vy,const QImage& frame1,const QImage& frame2);
	void getSelectedMotion(int frameIndex,QImage& patch1,QImage& patch2,DImage& vx,DImage& vy,const QImage& frame1,const QImage& frame2);
	void getLayerPatch(int frameIndex,QImage& patch,const QImage& frame);

	// functions for editing correspondence
	void addCorrespondence(int frameIndex,const QPointF& point1,const QPointF& point2);
	void addCorrespondence(int frameIndex,const QPointF& point1,const SortQueue<Point>& sort);
	bool IsParametricMotion(int frameIndex) const;
	void estimateMotion(int frameIndex);
	void setCorrespondenceFlowDimension(int frameIndex);
	inline Correspondence& correspondence(int frameIndex){return m_CorrespondenceList[frameIndex];};
	inline const Correspondence& correspondence(int frameIndex) const {return m_CorrespondenceList[frameIndex];};

	//stroke functions
	void createStroke(int frameIndex,const QPointF& point,StrokeProperties& sp,float pressure,int strokeId);
	void addStrokePoint(int frameIndex,const QPointF& point,StrokeProperties& sp, double ScaleFactor,float pressure=1);
	void setStrokeProperties(int frameIndex, StrokeProperties& sp);
	void setSelectedStrokeProperties(int frameIndex, QString prop, QString val);
	void finishedStroke(int frameIndex,bool showColorStroke=true, bool showGuideStroke=true, bool showSeedStroke=true,bool showRegularStroke=true);
	void renderStrokes(int frameIndex);
	void undoStroke(int strokeId);
	void deleteCurrentStrokeOneDirection(int frameIndex, bool forward, int deleteId);
	void deleteCurrentStroke(int frameIndex, int deleteId, int startDeleteFrame, int endDeleteFrame, int *firstFrame, int *lastFrame);
	void undoDeletedStrokes(int m_CurrDeleteId);
	void deleteUserStrokes(int frameIndex, int deleteId);
	void deleteAllUserStrokes();
	void deleteStrokeImages(int frameIndex);
	void deleteAllGeneratedStrokes();
	void refreshAllStrokeImages();
	StrokeProperties& getSelectedStrokeProperties(int frameIndex);
	void regularizeContour(const ContourLabel& lastFrameContour,ContourLabel& currFrameContour,float prevStrokeFactor, float flowFactor);
	void undoGeneratedStrokes(int generateId);
	void saveGeneratedStrokes(int curFrameIndex,QVector<Stroke *>& generatedStrokes,int generateId);
    QMap<int,QVector<Stroke *> >& getGeneratedStrokes(int curFrameIndex){return m_GeneratedStrokeList[curFrameIndex];};
	void saveGeneratedStrokesParameters(int generateId,StrokeLayerProperties& slp);
	void redrawGeneratedStrokes(int curFrameIndex);
	

	void createStrokeKDTrees();
	void createStrokeKDTree(int curFrameIndex,bool guideTree);
	void createBoundaryKDTree(int curFrameIndex);
	inline ANNkd_tree* getGuideKDTree(int curFrameIndex){return m_GuideStrokeKDTreeList[curFrameIndex];};
	inline ANNkd_tree* getColorKDTree(int curFrameIndex){return m_ColorStrokeKDTreeList[curFrameIndex];};
	inline ANNkd_tree* getBoundaryKDTree(int curFrameIndex){return m_BoundaryKDTreeList[curFrameIndex];};

	//getter and setter for the stroke images and associated textures
	QImage& getStrokeImage(int frameIndex);
	void setStrokeImage(int frameIndex,QImage& img, int generateId);
	inline int getStrokeImageTexture(int frameIndex){return m_StrokeImageTextureList[frameIndex];};
	inline void setStrokeImageTexture(int frameIndex,int tex){m_StrokeImageTextureList[frameIndex] = tex;};
	inline void setStrokeImageState(int frameIndex, int state){m_StrokeImageStateList[frameIndex] = state;};
	inline int getStrokeImageState(int frameIndex){return m_StrokeImageStateList[frameIndex];}; 
	inline bool strokeImageExists(int frameIndex){return ((m_StrokeImageStateList[frameIndex]==TEXTURE_EXISTS)||(m_StrokeImageStateList[frameIndex]==CREATE_TEXTURE));};
	inline int getStrokeBorderSize(){return m_StrokeBorderSize;};
	inline float getStrokeImageBoundaryBlur(){return m_StrokeBoundaryBlur;};
	inline float getStrokeImageMedianBlur(){return m_StrokeMedianBlur;};
	
	inline void setStrokeImageBoundaryBlur(float boundaryBlur){m_StrokeBoundaryBlur = boundaryBlur;};
	inline void setStrokeImageMedianBlur(int medianBlur){m_StrokeMedianBlur = medianBlur;};

	void shiftSelectedStrokes(int curFrameIndex, QPointF shift);
	void pushSelectedStrokes(int curFrameIndex, int strokeWidth, QPointF& p1,QPointF& p2);
	void undoChangedStrokes(int actionId);
	void propagatePushedPoints(int curFrameIndex, int actionId);
	void saveChangedStrokes(int curFrameIndex, int actionId,bool checkIfMoved, int saveStrokeId=-1);
	void checkStrokeLinks();
	void checkForDuplicateStokes();
	void  checkForSelectionProblems();

	void deleteStroke(int frame, int stroke);
	void fixSelectedStrokes(int curFrameIndex);
	void selectMultipleContours(int curFrameIndex, QPointF point, int strokeWidth, bool selecting);
	void deselectAllStrokes();
	inline void deselectCurrentStroke(){m_CurrentStroke =-1;};
	void deleteSelectedStrokes(int curFrameIndex, int deleteId,int startDeleteFrame, int endDeleteFrame, int *firstFrame, int *lastFrame);

	bool copySelectedStrokes(int curFrameIndex);
	int getSelectedStrokes(int curFrameIndex,QList<QList<ContourLabel> >& selectedStrokes);
	void addNewSelectedStrokes(QList<QList<ContourLabel> >& selectedStrokes);

	void setStrokeImagePainter(int frameIndex);
	void eraseUserStrokes(int curFrameIndex, QPointF& p1,QPointF& p2, int strokeWidth);

	bool applyOverdrawStroke(int curFrameIndex,QList<QImage>& frameList, double frameUpdate,int actionId, QVector<QPointF>& strokePoints,float imageScaleFactor, float strokeScalingFactor=0.5, bool redrawStroke=false);

	int maxGenerateStrokeId();

	void showStrokeTypes(bool showColorStroke, bool showGuideStroke, bool showSeedStrokes, bool showRegularStroke);

	void printGlobalLabels();




private:
	static inline double pabs(const QPointF& point){return sqrt(point.x()*point.x()+point.y()*point.y());};

};

#endif
