#include <QtGui>
#include "MagViewer.h"
#include "Correspondence.h"

MagViewer::MagViewer(QWidget *parent , Qt::WFlags flags): QWidget(parent,flags)
{
	m_WinSize=45;
	m_Interval=8;
	x0=y0=8;
	// set the possible scales
	m_ScaleList.append(1);
	m_ScaleList.append(2);
	m_ScaleList.append(6);

	reset();
	
	int winLength=m_WinSize*2+1;
	setFixedSize(winLength*3+m_Interval*2+x0*2,m_ScaleList.size()*(winLength+m_Interval)+y0*2-5);

	setWindowTitle(tr("Magnifier"));
	timer.start(800,this);
	steps=0;
}


MagViewer::~MagViewer(void)
{
}

//----------------------------------------------------------------------------------------------------
// function to reset the magnifier viewer
//----------------------------------------------------------------------------------------------------
void MagViewer::reset()
{
	int winLength=m_WinSize*2+1;
	QImage temp(winLength,winLength,QImage::Format_RGB32);
	QPainter painter(&temp);
	QBrush brush(QColor(200,200,200));
	//painter.setBrush(brush);
	painter.fillRect(0,0,winLength,winLength,brush);

	m_ImageList1.clear();
	m_ImageList2.clear();
	for(int i=0;i<m_ScaleList.size();i++)
	{
		m_ImageList1.append(temp);
		m_ImageList2.append(temp);
	}
	setFixedWidth(winLength*3+m_Interval*2);
	setFixedHeight(m_ScaleList.size()*winLength+(m_ScaleList.size()-1)*m_Interval);
}


void MagViewer::loadImages(const QImage* refImage1,const QImage* refImage2)
{
	pRefImage1=refImage1;
	pRefImage2=refImage2;
}

void MagViewer::GenerateSubImages()
{
	for(int i=0;i<m_ImageList1.size();i++)
	{
		Correspondence::getPatch(m_ImageList1[i],*pRefImage1,m_Point1.x(),m_Point1.y(),m_WinSize,(double)1/m_ScaleList[i]);
		Correspondence::getPatch(m_ImageList2[i],*pRefImage2,m_Point2.x(),m_Point2.y(),m_WinSize,(double)1/m_ScaleList[i]);
	}
}

void MagViewer::updatePoints(const QPointF& point1,const QPointF& point2)
{
	m_Point1=point1;
	m_Point2=point2;
	GenerateSubImages();
	update();
}

void MagViewer::paintEvent(QPaintEvent *event)
{
	QPainter mPainter(this);
	mPainter.setPen(QColor(0,0,0));

	int winLength=m_WinSize*2+1;
	QPen pen;
	pen.setWidthF(1);
	int x,y;

	for(int i=0;i<m_ImageList1.size();i++)
	{
		// draw images
		mPainter.drawImage(QPoint(x0,y0+i*(winLength+m_Interval)),m_ImageList1[i]);
		mPainter.drawImage(QPoint(x0+winLength+m_Interval,y0+i*(winLength+m_Interval)),m_ImageList2[i]);
		if(steps%2==0)
			mPainter.drawImage(QPoint(x0+(winLength+m_Interval)*2,y0+i*(winLength+m_Interval)),m_ImageList1[i]);
		else
			mPainter.drawImage(QPoint(x0+(winLength+m_Interval)*2,y0+i*(winLength+m_Interval)),m_ImageList2[i]);

		// draw rectangles
		mPainter.setPen(Qt::black);
		mPainter.drawRect(x0,y0+i*(winLength+m_Interval),winLength,winLength);
		mPainter.drawRect(x0+winLength+m_Interval,y0+i*(winLength+m_Interval),winLength,winLength);
		mPainter.drawRect(x0+(winLength+m_Interval)*2,y0+i*(winLength+m_Interval),winLength,winLength);

		// draw cross
		pen.setColor(Qt::red);
		x=x0+m_WinSize+1;
		y=y0+i*(winLength+m_Interval)+m_WinSize+1;
		
		mPainter.setPen(pen);
		mPainter.drawLine(x-5,y,x+5,y);
		mPainter.drawLine(x,y-5,x,y+5);

		pen.setColor(Qt::green);
		x=x0+winLength+m_Interval+m_WinSize+1;
		y=y0+i*(winLength+m_Interval)+m_WinSize+1;
		
		mPainter.setPen(pen);
		mPainter.drawLine(x-5,y,x+5,y);
		mPainter.drawLine(x,y-5,x,y+5);

	}
}

void MagViewer::timerEvent(QTimerEvent *event)
 {
     if (event->timerId() == timer.timerId()) {
         ++steps;
		update();
     } else {
         QWidget::timerEvent(event);
	 }
} 