#include "ReduceFrameRange.h"

ReduceFrameRange::ReduceFrameRange(QWidget* parent,int numFrames): QDialog(parent)
{
	m_NumFrames=numFrames;
	m_StartFrame=1;
	m_EndFrame=m_NumFrames;

	startLabel=new QLabel("Start frame: ");
	endLabel=new QLabel("End frame:");
	startFrameSpinBox=new QSpinBox();
	endFrameSpinBox=new QSpinBox();
	startFrameSpinBox->setMaximum(m_NumFrames-1);
	startFrameSpinBox->setMinimum(1);
	startFrameSpinBox->setValue(m_StartFrame);
	endFrameSpinBox->setMaximum(m_NumFrames);
	endFrameSpinBox->setMinimum(2);
	endFrameSpinBox->setValue(m_EndFrame);

	connect(startFrameSpinBox,SIGNAL(valueChanged(int)),this,SLOT(startFrameChanged(int)));
	connect(endFrameSpinBox,SIGNAL(valueChanged(int)),this,SLOT(endFrameChanged(int)));
	
	layoutSpin=new QHBoxLayout;
	layoutSpin->addWidget(startLabel);
	layoutSpin->addWidget(startFrameSpinBox);
	layoutSpin->addSpacing(10);
	layoutSpin->addWidget(endLabel);
	layoutSpin->addWidget(endFrameSpinBox);

	doneButton=new QPushButton("Done");
	cancelButton=new QPushButton("Cancel");
	doneButton->setFixedWidth(60);
	cancelButton->setFixedWidth(60);

	connect(doneButton,SIGNAL(clicked()),this,SLOT(accept()));
	connect(cancelButton,SIGNAL(clicked()),this,SLOT(reject()));

	layoutButton=new QHBoxLayout;
	layoutButton->addSpacing(70);
	layoutButton->addWidget(doneButton);
	layoutButton->addWidget(cancelButton);

	layout=new QVBoxLayout;
	layout->addLayout(layoutSpin);
	layout->addSpacing(6);
	layout->addLayout(layoutButton);

	setLayout(layout);

	QString title("Reduce frame range (1~");
	title+=QString::number(m_NumFrames);
	title+=")";
	setWindowTitle(title);
}

ReduceFrameRange::~ReduceFrameRange(void)
{
}

void ReduceFrameRange::startFrameChanged(int startFrame)
{
	m_StartFrame=startFrame;
	endFrameSpinBox->setMinimum(m_StartFrame+1);
}

void ReduceFrameRange::endFrameChanged(int endFrame)
{
	m_EndFrame=endFrame;
	startFrameSpinBox->setMaximum(m_EndFrame-1);
}