#ifndef _VideoLabeling_h
#define _VideoLabeling_h


#include <QObject>
#include <QString>
#include <QList>
#include <QPainter>
#include "VideoSequence.h"
#include "LayerObject.h"
#include "ContourLabel.h"
#include "ImageViewer.h"
#include "MainWindow.h"
#include "LayerDepth.h"
#include "XmlWriter.h"
#include "MotionPara.h"
#include "ImPyramid.h"
#include "MaskPyramid.h"
#include "StrokeProperties.h"
#include "StrokeLayerProperties.h"


class VideoSequence;
class LayerObject;
class ContourLabel;
class MainWindow;
class LayerDepth;
class XmlWriter;

class VideoLabeling: public VideoSequence
{
	Q_OBJECT

	// the declaimation of state
public:
	enum LabelState{Idle,CursorOver,CursorOverStroke,CreateContour,CreateStroke,Selected,MovePoint,ScaleContour,MoveContour,ModifySecond,ColorPicker,StrokeEraser,SelectMultipleStrokes,MoveSelectedStrokes,PushSelectedStrokes,OverdrawStroke};

	// member variablse
private:
	enum Selected{Nothing,BoundaryContour,StrokeContour};
	enum Mode{SelectMode,CreateStrokeMode,CreateLayerMode};
	enum Action{DeleteStrokesAction,CreateStrokeAction,GenerateStrokesAction,ChangeStrokeAction,CreateRegionAction};
	QList<int> m_ActionHistory;

	int m_CurrentMode;


	QVector<QPointF> m_SelectPoints;
	bool m_SelectingPoints;

	int _strokeWidth;

	// version member
	QString m_Version;
	static double m_CurVersion;

	QList<LayerObject> m_LayerObjectList;
	int selectedLayerIndex;

	// variables for labeling
	LabelState labelState;  // variable of labeling state
	int selectedBoundingBoxCorner;

	MainWindow* pMainWindow;

	// variable to handle UI control
	bool IsMouseAllowed;

	// variable to indicate whether background layer has been created
	bool IsBgLayerCreated;
	bool IsLayerMaskGenerated;

	// the members of pyramid
	QList<ImPyramid> m_ImPyramidList;
	bool m_IsPyramidCreated;

	// the parameters of labeling correspondence
	// they will not be stored in the data
	int tabIndex;
	bool IsEditingCorrespondence;
	int m_SearchSize;
	int m_WinSize;
	QImage m_NextImage;
	bool m_IsUncertainMatching;
	bool m_IsManualMatching;
	QPointF m_PrePoint;

	MaskPyramid m_MaskPyramid;
	bool IsMaskPyramidComputed;


	//current stroke properties
	StrokeProperties m_CurrentStrokeProperties;
	int m_RedSum, m_GreenSum, m_BlueSum, m_ColorCount;
	int m_MaxStrokeId, m_ChangeStrokeId, m_GeneratedStrokesId;
	QList<int> m_GeneratedStrokeLayerList;


	int _hueOffset, _saturationOffset, _valueOffset;
	double _hueNoise, _saturationNoise, _valueNoise;
	double _colorUpdate;

	float _boundaryFlowFactor;
	float _imageFlowFactor;
	float _userFlowFactor;
	float _fixedFlowFactor;
	float _fixedMovementAngle;
	int _fixedMovementRate; 

	float _followPreviousStrokesFactor, _followGradientFactor;

	//what to display
	bool m_ShowStrokes;
	bool m_ShowStrokeSpines;
	bool m_ShowLayerBoundaries;
	bool m_ShowSeedMask;
	bool m_ShowColorStrokes;
	bool m_ShowGuideStrokes;
	bool m_ShowSeedStrokes;
	bool m_ShowRegularStrokes;

	float m_CurrentBoundaryBlur;
	int m_CurrentMedianBlur;
	bool m_IsLoadingStrokeProperties;
	bool IsBoundarySelected;

	int m_CurrDeleteId;

	int m_FillFramesForward, m_FillFramesBackward;

	QCursor m_MultipleSelectCursor;
	QCursor m_DrawCursor;
	QCursor m_OverdrawCursor;
	QCursor m_PushCursor;
	QCursor m_CreateRegionCursor;

	QString m_ProjectDir;

	char _strbuf[2048];

	int m_OverdrawStrokeFromLayer;

	double normal(const double &mean, const double &std);
	double truncNormal(const double &mean, const double &std,const double &lowerBound,const double &upperBound);

	void deselectAllStrokes();
	int getNearestLayer(int curFrameIndex, QPointF point);
	void showStrokeTypes();

	// member functions
public:
	VideoLabeling(MainWindow* parent);
	VideoLabeling(const VideoLabeling& videoLabeling);
	void copyData(const VideoLabeling& videoLabeling);
	VideoLabeling& operator=(const VideoLabeling& videoLabeling);
	~VideoLabeling(void);
	
	void clear();
	static double curVersion() {return m_CurVersion;};
	void Initialize(const QString& path,const QFileInfoList& filelist);
	void OutputLabelFrame(int frameIndex,const QString& filename,int quality=95);
	void OutputAllLabeling(const QString& pathname);
	// function to help handle mouse events (operating the list of LayerObjects)
	bool selectLayerObject(const QPointF& point,double ScaleFactor=1);
	bool selectLayerObjectFromInterior(const QPointF &point);
	void setupSelectedLayer();
	void createLayerObject(const QPointF& point);
	void startStroke(const QPointF& point,float pressure, double ScaleFactor=1);
	bool selectStroke(const QPointF& point,double ScaleFactor=1);
	bool selectPoint(const QPointF& point,double ScaleFactor=1,bool boundaryContour=true);
	int selectPopupMenu(bool IsPointSelected);
	void ProcessPopupMenuSelection(int choice);

	QString projectDir(){return m_ProjectDir;};
        void createDirectoryStructure(const QString& filename,bool updateFramePath=true);
	bool writeVideoLabelMeXML(const QString& filename);
	bool readVideoLabelMeXML(const QString& filename);
	// function for replicating
	void ReplicateContour(bool direction,int destFrame);
	// function for tracking
	void TrackContour(bool direction,int nFrames);
	void OccludingReasoning(QList<bool>& IsVisibleList,const QList<QPointF>& contour);
	bool OccludingReasoning(const QPointF& point);
	void propagateUpdatePoint();
	void TrackPoint(int pointIndex,bool direction,int nFrames,int dstFrame);
	int findLastLocalLabeledFrame(int frameIndex,int pointIndex,bool &IsLastFrameLabeled);
	int findFirstLocalLabeledFrame(int frameIndex,int pointIndex,bool &IsFirstFrameLabeled);
	
	// function for layering and motion analysis
	void outputLayerMask(QString& pathname);
	void outputVisibleLayerMask(QString& pathname);
	void outputDepthMap(QString& pathname);
	void getDepthMap(int frame, Image<unsigned char>& img);
	void outputMotion(QString& pathname);
	void outputLayerMotion(QString& pathname);
	bool isLayerMaskGenerated() const {return IsLayerMaskGenerated;};

	// functions for file IO
	bool writeVideoLabeling(const QString& filename);
	bool readVideoLabeling(const QString& filename);

	bool copyVideoLabelingFiles(const QString &oldDir,const QString &newDir);
	
	// function to reduce the range of the frames
	void reduceFrameRange(int startFrame,int endFrame);
	void buildPyramid();

	void renderPainting(int requestStartFrame,int requestEndFrame,float renderScale,QString outputDir);
signals:
	void changeLayerDepth(const QList<LayerDepth>& layerDepthList,int activeLayerIndex,int frame_index);
	void loadLayerMotionPara(const MotionPara&);
	void layerDeSelected();
	void setMotionAnalysis(bool);
	void motionSelected(const QImage& patch1,const QImage& patch2,const DImage& vx,const DImage& vy);
	void motionSelected(const QImage& patch1,const QImage& patch2);
	void motionSelected(const QImage& patch1);
	void SearchingWindowSizeChanged(int searchSize,int winSize);
	void motionTypeChanged(int index);
	void setRow(int row);
	void setTab(int index);
	void frameChanged(const QImage* image1,const QImage* image2);
	void pointSelected(const QPointF& point1,const QPointF& ponit2);
	void lambdaChanged(double lambda);
	void paintStrokes(ImageViewer& imageViewer,QPainter& painter,QList<QImage>& img,QList<LayerObject>& LayerObjectList,int curFrameIndex);
	void strokesChanged(int currFrame);
	void redrawStrokes();
	void generateStrokes(QString& path,LayerObject& layer, QList<LayerObject>& m_LayerObjectList, QList<QImage>& img,QStringList& fileList,int currFrame, int generateId,StrokeLayerProperties *slp,bool fromScript);
	void renderStrokes(int requestStartFrame,int requestEndFrame,float renderScale, QList<LayerObject>& m_LayerObjectList, QList<QImage>& img,QString outputDir);
	
	void updatePropertiesForSelectedStroke(StrokeProperties& prop);
	void showUserStrokeOptions(bool);
	void setCurrentColor(QColor& color, float pointx, float pointy);
	void setCurrentColorForFilledStroke(Stroke* stroke, QImage& img);

public slots:
	void ProcessMouseEvent(ImageViewer& imageViewer,ImageViewer::MouseAction& mouseAction,QPoint& Pos,float pressure,Qt::KeyboardModifiers& km);
	void Paint(ImageViewer& imageViewer,QPainter& painter,QImage& saveimg);
	void gotoFrame(int findex);
        void setLayerDepth(const LayerDepth& layerDepth);
	void GenerateLayers();
	void updateLayerMotionPara(const MotionPara& mPara){
		m_LayerObjectList[selectedLayerIndex].LayerMotionPara()=mPara;
	}
	void MotionAnalysis();
	void motionParaRowSelected(int row);
	void motionParaRowSelected();
	void tabChanged(int index);
	void setSearchSize(int searchSize){m_SearchSize=searchSize;};
	void setWinSize(int winSize){m_WinSize=winSize;};
	void setManualMatching(bool isManualMatching) {m_IsManualMatching=isManualMatching;};
	void setUncertainMatching(bool _isUncertainMatching) {m_IsUncertainMatching=_isUncertainMatching;};
	void setParametricMotionType(int motionType);
	void setLambda(double lambda);
	void clearLabeling();
	void redoMatching();
	void grabFeatures();

	void deselectBoundary(){if (labelState==Selected)labelState=Idle;};

	void selectContourMode(){m_CurrentMode=SelectMode; labelState=Idle; QApplication::setOverrideCursor(QCursor(Qt::ArrowCursor));emit Update();};
	void createStrokeMode(){emit showUserStrokeOptions(true);m_CurrentMode=CreateStrokeMode;labelState=Idle;  QApplication::setOverrideCursor(QCursor(m_DrawCursor));emit Update();};
	void createLayerMode(){emit showUserStrokeOptions(false);m_CurrentMode=CreateLayerMode;labelState=Idle;  QApplication::setOverrideCursor(QCursor(m_CreateRegionCursor));emit Update();};
	void overdrawContourMode(){labelState=OverdrawStroke;  QApplication::setOverrideCursor(QCursor(m_OverdrawCursor));emit Update();};
	
	void pickColorMode(){labelState=ColorPicker; QApplication::setOverrideCursor(QCursor(Qt::CrossCursor));emit Update();};
	void selectMultipleStrokeMode(){emit showUserStrokeOptions(true);labelState=SelectMultipleStrokes; QApplication::setOverrideCursor(m_MultipleSelectCursor);emit Update();};
	void moveSelectedStrokes(){labelState=MoveSelectedStrokes; QApplication::setOverrideCursor(QCursor(Qt::SizeAllCursor));emit Update();};
	void pushSelectedStrokes(){labelState=PushSelectedStrokes; QApplication::setOverrideCursor(QCursor(m_PushCursor));emit Update();};


	void setStrokeWidth(int width);
	void deleteSelectedStrokes();
	void deleteSelectedStrokesForward();
	void deleteSelectedStrokesBackward();
	void deleteSelectedStrokes(bool forward, bool backward);
	void generateStrokeLayer();
        void generateStrokeLayersFromList(QString filename=QString());
	void deleteStrokeLayers();
	void undoLastLayer();
	void changeRegionBoundaryBlur();
	void changeRegionMedianBlur();
	void replicateSelectedStrokes();
	void selectAllRegionStrokes();
	void assignStrokesToRegion();
	void fixSelectedStrokes();
	void undoLastAction();



	void userFlowFactorChanged(int userFlowFactor){_userFlowFactor = userFlowFactor;};
	void imageFlowFactorChanged(int imageFlowFactor){_imageFlowFactor = imageFlowFactor;};
	void boundaryFlowFactorChanged(int boundaryFlowFactor){_boundaryFlowFactor = boundaryFlowFactor;}
	void fixedFlowFactorChanged(int fixedFlowFactor){_fixedFlowFactor = fixedFlowFactor;}
	void fixedMovementDirectionChanged(int fixedMovementAngle){_fixedMovementAngle = ((double)(360+(-1*fixedMovementAngle-90)%360) * (3.1415927/180.0));};
	void fixedMovementRateChanged(int fixedMovementRate){_fixedMovementRate = fixedMovementRate;};
	
	void followStrokesFactorChanged(double followStrokesFactor){_followPreviousStrokesFactor = followStrokesFactor;};
	void followGradientFactorChanged(double followGradientFactor){_followGradientFactor = followGradientFactor;};


	void showStrokeSpines(){if (m_ShowStrokeSpines) m_ShowStrokeSpines = false; else m_ShowStrokeSpines = true;emit Update();};
	void showStrokes(){if (m_ShowStrokes) m_ShowStrokes = false; else m_ShowStrokes = true;emit Update();};
	void showLayerBoundaries(){if (m_ShowLayerBoundaries) m_ShowLayerBoundaries = false; else m_ShowLayerBoundaries = true;emit Update();};

	void showColorStrokes(bool showColorStrokes){m_ShowColorStrokes=showColorStrokes;showStrokeTypes(); };
	void showGuideStrokes(bool showGuideStrokes){m_ShowGuideStrokes=showGuideStrokes;showStrokeTypes(); };
	void showSeedStrokes(bool showSeedStrokes){m_ShowSeedStrokes=showSeedStrokes;showStrokeTypes();};
	
	void showRegularStrokes(bool showRegularStrokes){m_ShowRegularStrokes=showRegularStrokes;showStrokeTypes(); emit Update();};

	void updateSelectedStrokes(QPointF point,bool deselecting);

	//painting slots
	void setStrokeProperties(StrokeProperties& sp);
	void setStrokeProperties(QString prop, QString val);
	void fillFramesBackwardChanged(int fillFramesBackward){m_FillFramesBackward = fillFramesBackward;};
	void fillFramesForwardChanged(int fillFramesForward){m_FillFramesForward = fillFramesForward;};
	void boundaryBlurChanged(double boundaryBlur){printf("boundaryBlur %f\n", boundaryBlur);m_CurrentBoundaryBlur = boundaryBlur;};
	void medianBlurChanged(int medianBlur){printf("medianBlur %i\n", medianBlur);m_CurrentMedianBlur = medianBlur;};

	void hueOffsetChanged(int hueOffset){printf("hueOffsetChanged %i\n",hueOffset);_hueOffset = hueOffset;};
	void saturationOffsetChanged(int saturationOffset){printf("saturationOffsetChanged %i\n",saturationOffset);_saturationOffset = saturationOffset;};
	void valueOffsetChanged(int valueOffset){printf("valueOffsetChanged %i\n",valueOffset);_valueOffset = valueOffset;};
	void hueNoiseChanged(double hueNoise){printf("hueNoiseChanged %f\n",hueNoise);_hueNoise = hueNoise;};
	void saturationNoiseChanged(double saturationNoise){printf("saturationNoise %f\n",saturationNoise);_saturationNoise = saturationNoise;};
	void valueNoiseChanged(double valueNoise){printf("valueNoiseChanged %f\n",valueNoise);_valueNoise = valueNoise;};
	

	void colorUpdateChanged(double colorUpdate){printf("color update %f\n", colorUpdate);_colorUpdate = colorUpdate;};
	

};

#endif
