
#include "VideoSequence.h"
#include <iostream>
using namespace std;

VideoSequence::VideoSequence(void): QObject()
{
	IsVideoLoaded=false;
	framesRead=false;
	curFrameIndex=0;
	numFrames=0;
}

VideoSequence::~VideoSequence(void)
{
}

//-------------------------------------------------------------------------------------------
// the function to copy the data from another instance
//-------------------------------------------------------------------------------------------
void VideoSequence::copyData(const VideoSequence& videoSequence)
{
	printf("copying video sequence\n");
	framePath=videoSequence.framePath;
	fileList=videoSequence.fileList;
	m_Image=videoSequence.m_Image;
	curFrameIndex=videoSequence.curFrameIndex;
	numFrames=videoSequence.numFrames;
	frameList = videoSequence.frameList;
}

VideoSequence::VideoSequence(const VideoSequence& videoSequence)
{
	copyData(videoSequence);
}

void VideoSequence::clear()
{
	framePath.clear();
	fileList.clear();
	numFrames=0;
	curFrameIndex=0;
}

void VideoSequence::Initialize(const QString& path,const QFileInfoList& filelist)
{
	framePath=path;
	fileList.clear();
	for(int i=0;i<filelist.size();i++)
		fileList.append(filelist[i].fileName());

	numFrames=fileList.size();
	curFrameIndex=0;
	gotoFrame(curFrameIndex+1);
	IsVideoLoaded=true;
	emit NumFramesChanged(numFrames);
}

//----------------------------------------------------------------------------------------------------------------------
// function to load the current frame according to curFrameIndex
//----------------------------------------------------------------------------------------------------------------------
bool VideoSequence::loadFrame()
{
	//load all the frames if necessary
	if (framesRead==false)
	{
		if (readInFrames() == false)
			return false;
	}

	if(curFrameIndex<0 || curFrameIndex>=numFrames)
		return false;

	m_Image = frameList[curFrameIndex];



	emit ImageChanged(m_Image);
	return true;
}

//----------------------------------------------------------------------------------------------------------------------
// function to read in all the frames
//----------------------------------------------------------------------------------------------------------------------
bool VideoSequence::readInFrames()
{
	if (framesRead ==true)
		return true;

    if (fileList.size() < numFrames)
    {
       printf("filelist size %i didn't match numFrame %i\n", fileList.size(),numFrames);
       return false;

    }
	for(int i=0;i < numFrames;i++)
	{
		QImage img;
		QString file(framePath+fileList[i]);
        cout << "Reading in frame " << file.toAscii().data() << std::endl;
		
		if(img.load(framePath+fileList[i])==false)
        {
            cout << "Problem loading " << std::endl;
			return false;
        }


		frameList.push_back(img);
	}

	framesRead = true;
    IsVideoLoaded=true;
	return true;
}

//----------------------------------------------------------------------------------------------------------------------
// function to set the control of frame indexing
//----------------------------------------------------------------------------------------------------------------------
void VideoSequence::updateFrameControl()
{
	if(curFrameIndex==0)
		emit FrameControlChanged(false,false,true,true);
	else if(curFrameIndex==numFrames-1)
		emit FrameControlChanged(true,true,false,false);
	else
		emit FrameControlChanged(true,true,true,true);
	emit FrameIndexChanged(curFrameIndex+1);
}

//----------------------------------------------------------------------------------------------------------------------
// Goto frame findex. Note that findex ranges [1,numFrames] instead of C++ way [0,numFrames-1] so
// that other widgets can directly control change of frames. So increase the frame number by 1 before
// calling this function
//----------------------------------------------------------------------------------------------------------------------
void VideoSequence::gotoFrame(int findex)
{
	findex--;
	if(findex<0 || findex>=numFrames)
		return;
	curFrameIndex=findex;
	loadFrame();
	updateFrameControl();
	emit Update();
	//m_Image.save("test.jpg");
}

void VideoSequence::gotoFirstFrame()
{
	if(curFrameIndex==0)
		return;
	gotoFrame(1);
}

void VideoSequence::gotoPreFrame()
{
	if(curFrameIndex==0)
		return;
	gotoFrame(curFrameIndex);
}

void VideoSequence::gotoNextFrame()
{
	if(curFrameIndex==numFrames-1)
		return;
	gotoFrame(curFrameIndex+2);
}

void VideoSequence::gotoLastFrame()
{
	if(curFrameIndex==numFrames-1)
		return;
	gotoFrame(numFrames);
}




