#include <QtGui/QApplication>
#include <QList>
#include <QString>
#include <QDir>
#include <QDateTime>
#include <iostream>
#include "MainWindow.h"
#include "DepthControl.h"
#include "LayerDepth.h"
#include "Image.h"
#include "LinearSystem.h"
#include "BinaryImage.h"
#include "FlowFieldViewer.h"
#include "MagViewer.h"
#include "tri_logger.hpp"

void DepthMap(int width=10);


void myMessageOutput( QtMsgType type, const char *msg )
{
	switch ( type ) 
	{
		case QtDebugMsg:
			fprintf( stderr, "Debug: %s\n", msg );
			break;
		case QtWarningMsg:
			fprintf( stderr, "Warning: %s\n", msg );
			break;
		case QtFatalMsg:
			fprintf( stderr, "Fatal: %s\n", msg );
			//break;
			abort();                    // deliberately core dump
	}
}


int main(int argc, char *argv[])
{

	//qInstallMsgHandler( myMessageOutput );
    Application a(argc, argv);

	

     #ifdef Q_WS_MAC
         QDir dir(QApplication::applicationDirPath());
         dir.cdUp();
         dir.cd("plugins");
         QApplication::setLibraryPaths(QStringList(dir.absolutePath()));
         QApplication::addLibraryPath(dir.absolutePath()+QString("/imageformats"));
         QString pth = dir.absolutePath();
         cout <<"image format location ";
         cout << pth.toAscii().data() << endl;
    #endif

     #ifdef Q_WS_WIN
        QString path=QDir::currentPath();
        if (path.right(1)!="/")
            path+="/";

		QDir currDir(path);
		currDir.mkdir(QString("logs"));


		path += "imageformats";
		QApplication::addLibraryPath(path);
     #endif


	QGLFormat f;
	f.setAlpha(true);
	f.setAlphaBufferSize(1);
	f.setStencil(true);
	QGLFormat::setDefaultFormat( f );


	QString currtime =QDateTime::currentDateTime().toString();
					
	char strbuf[2048];
    sprintf(strbuf,"Started at %s",currtime.toAscii().data() );
    LOG_STR_COUT(strbuf);


	MainWindow w;
    w.show();

	for(int i = 0; i < argc; i++) 
	{

		cout << "argv[" << i << "] = " <<  argv[i]<< endl ;
		QString arg(argv[i]);
      
		if (arg.contains(QString(".xml"))==true)
		{
			w.runScript(arg);
		}
	}

    a.connect(&a, SIGNAL(lastWindowClosed()), &a, SLOT(quit()));
    return a.exec();
}




void DepthMap(int width)
{
	BiImage image(width,256,3);
	unsigned char* pData=image.data();
	QColor color;
	int RGB[3];

	for(int i=0;i<256;i++)
	{
		color.setHsv(i,255,255);
		RGB[0]=color.red();
		RGB[1]=color.green();
		RGB[2]=color.blue();
		for(int j=0;j<width;j++)
			for(int k=0;k<3;k++)
				pData[(i*width+j)*3+k]=RGB[k];
	}
	image.imwrite("DepthMap.bmp");
}
